/*
  htmlencode.c
  
   1  2006-03-18  Matlib 
   2  2009-02-22  Matlib  + link against libarguments
                          + add <br> support
      2011-02-16  Matlib  update to libarguments v. 3
*/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <arguments.h>

#define BUFFER_SIZE 128

#define CHECK_ERROR(__COND__,__FUNC__) \
  { \
    if (__COND__) \
    { \
      perror (__FUNC__); \
      exit (EXIT_FAILURE); \
    } \
  }
     


struct Argument arguments [] =
{
  {'b', "br",         argument_bool, ARGUMENT_F_ONCE},
  {'B', "xml-br",     argument_bool, ARGUMENT_F_ONCE},
  {'c', "leave-cr",   argument_bool, ARGUMENT_F_ONCE},
  {'l', "no-lf",      argument_bool, ARGUMENT_F_ONCE},
  {'s', "spaces",     argument_bool, ARGUMENT_F_ONCE},
  {'S', "all-spaces", argument_bool, ARGUMENT_F_ONCE},
  {}
};

enum
{
  ARG_BR,
  ARG_BRX,
  ARG_CR,
  ARG_NNL,
  ARG_SPC,
  ARG_SPA
};

#define BR  (arguments [ARG_BR].value_u)
#define BRX (arguments [ARG_BRX].value_u)
#define CR  (arguments [ARG_CR].value_u)
#define NNL (arguments [ARG_NNL].value_u)
#define SPC (arguments [ARG_SPC].value_u)
#define SPA (arguments [ARG_SPA].value_u)







int main (int const argc, char const *const *const argv)
{
  char buffer [BUFFER_SIZE];
  char *enc_table [255];
  char char_table [2 * sizeof (enc_table)];
  char *fsp, *dbs;
  int i, c;
  
  i = 0;
  for (c = 0; c < 255; c ++)
  {
    enc_table [c] = char_table + i;
    char_table [i ++] = (char) c;
    char_table [i ++] = 0;
  }
  i = arguments_parse ("htmlencode", "2 (2009-02-22)", "[text]", 
     "Converts plain text characters to HTML/XML entities.",
     argc, argv, arguments);
  if (NNL)
    enc_table ['\n'] = "";
  if (BR)
  {
    if (BRX)
      arguments_error_freeform ("Can't use both XML and SGML entity forms.");
    if (NNL)
      enc_table ['\n'] = "<br>";
    else
      enc_table ['\n'] = "<br>\n";
  }
  if (BRX)
  {
    if (NNL)
      enc_table ['\n'] = "<br />";
    else
      enc_table ['\n'] = "<br />\n";
  }
  if (!CR)
    enc_table ['\r'] = "";
  if (SPC)
    dbs = "&nbsp;";
  else
    dbs = " ";
  if (SPA)
  {
    if (SPC)
      arguments_error_freeform ("Options -s and -S are incompatible.");
    fsp = dbs = "&nbsp;";
  }
  else
    fsp = " ";
  enc_table [' '] = fsp;
  enc_table ['"'] = "&quot;";
  enc_table ['&'] = "&amp;";
  enc_table ['<'] = "&lt;";
  enc_table ['>'] = "&gt;";
  
  if (i == argc)
  {
loop1:
    i = read (STDIN_FILENO, buffer, BUFFER_SIZE - 1);
    CHECK_ERROR (i < 0, "read");
    if (i == 0)
      return EXIT_SUCCESS;
    for (c = 0; c < i; c ++)
    {
      CHECK_ERROR (printf (enc_table [(unsigned char) buffer [c]]) < 0,
         "printf");
      if (buffer [c] == ' ')
        enc_table [' '] = dbs;
      else
        enc_table [' '] = fsp;
    }
    goto loop1;
  }
  
loop2:
  for (c = 0; argv [i][c] != 0; c ++)
  {
    CHECK_ERROR (printf (enc_table [(unsigned char) argv [i][c]]) < 0,
       "printf");
    if (argv [i][c] == ' ')
      enc_table [' '] = dbs;
    else
      enc_table [' '] = fsp;
  }
  i ++;
  if (i == argc)
    return EXIT_SUCCESS;
  CHECK_ERROR (printf (enc_table [' ']) < 0, "printf");
  enc_table [' '] = dbs;
  goto loop2;
}

